<?php

namespace App\Domain\Products\Observers;

use App\Domain\Kafka\Actions\Send\SendPublishedProductEventAction;
use App\Domain\Kafka\Messages\Send\ModelEvent\ModelEventMessage;
use App\Domain\Products\Models\PublishedProduct;

class PublishedProductAfterCommitObserver
{
    public bool $afterCommit = true;

    public function __construct(
        protected SendPublishedProductEventAction $sendPublishedProductEventAction
    ) {
    }

    public function created(PublishedProduct $model): void
    {
        if ($model->allow_publish) {
            // Созданные только что неопубликованные товары нет смысла отправлять
            $this->sendPublishedProductEventAction->execute($model, ModelEventMessage::CREATE);
        }
    }

    public function updated(PublishedProduct $model): void
    {
        if ($model->allow_publish || $model->wasChanged('allow_publish')) {
            // Отправляем только изменения по опубликованным товарам, или у которых изменился статус публикации
            // Нет смысла отправлять изменение по неопубликованным товарам, Старое состояние неопубликованного товара на витрине не критично
            $this->sendPublishedProductEventAction->execute($model, ModelEventMessage::UPDATE);
        }
    }

    public function deleted(PublishedProduct $model): void
    {
        $this->sendPublishedProductEventAction->execute($model, ModelEventMessage::DELETE);
    }
}
