<?php

namespace App\Http\ApiV1\Modules\Categories\Queries;

use App\Domain\Categories\Models\Property;
use Spatie\QueryBuilder\AllowedFilter;
use Spatie\QueryBuilder\QueryBuilder;

class PropertiesQuery extends QueryBuilder
{
    public function __construct()
    {
        parent::__construct(Property::query());

        $this->allowedSorts(['id', 'created_at', 'updated_at']);
        $this->defaultSort('id');

        $this->allowedFilters([
            AllowedFilter::exact('id'),
            AllowedFilter::exact('name'),
            AllowedFilter::exact('display_name'),
            AllowedFilter::exact('code'),
            AllowedFilter::exact('type'),
            AllowedFilter::exact('is_active'),
            AllowedFilter::exact('is_public'),
            AllowedFilter::exact('is_filterable'),
            AllowedFilter::exact('is_multiple'),
            AllowedFilter::exact('is_system'),
            AllowedFilter::exact('is_common'),
            AllowedFilter::exact('is_required'),
            AllowedFilter::exact('has_directory'),
            AllowedFilter::exact('hint_value'),
            AllowedFilter::exact('hint_value_name'),
            AllowedFilter::exact('created_at'),
            AllowedFilter::exact('updated_at'),

            AllowedFilter::partial('name_like', 'name'),
            AllowedFilter::partial('display_name_like', 'display_name'),
            AllowedFilter::partial('code_like', 'code'),
            AllowedFilter::scope('created_at_from'),
            AllowedFilter::scope('created_at_to'),
            AllowedFilter::scope('updated_at_from'),
            AllowedFilter::scope('updated_at_to'),
            AllowedFilter::partial('hint_value_like', 'hint_value'),
            AllowedFilter::partial('hint_value_name_like', 'hint_value_name'),

            AllowedFilter::exact('category_id', 'actualLinks.category_id'),
        ]);

        $this->allowedIncludes(['directory']);
    }
}
