<?php

use App\Domain\Categories\Models\Category;
use App\Domain\Categories\Models\CategoryPropertyLink;
use App\Domain\Categories\Models\Property;
use App\Domain\Categories\Models\PropertyType;
use App\Domain\Products\Models\Product;
use App\Domain\Products\Models\ProductPropertyValue;
use App\Http\ApiV1\Support\Tests\ApiV1ComponentTestCase;
use function Pest\Laravel\assertNotSoftDeleted;
use function Pest\Laravel\assertSoftDeleted;
use function Pest\Laravel\patchJson;
use function Pest\Laravel\postJson;

uses(ApiV1ComponentTestCase::class)->group('products', 'component', 'complex');

test('hide unbound attribute', function () {
    $category = Category::factory()->createOne();
    $product = Product::factory()->inCategory($category)->create();
    $property = Property::factory()
        ->withType(PropertyType::STRING)
        ->actual($category)
        ->createOne();

    CategoryPropertyLink::factory()->createFast($category, $property);
    $release = ProductPropertyValue::factory()
        ->forProduct($product)
        ->forProperty($property)
        ->published()
        ->createOne()
        ->release;

    $request = ['replace' => true, 'properties' => []];
    postJson("/api/v1/categories/categories/{$category->id}:bind-properties", $request)
        ->assertOk();

    assertSoftDeleted($release);
});

test('show bound attribute', function () {
    $category = Category::factory()->createOne();
    $product = Product::factory()->inCategory($category)->create();
    $property = Property::factory()
        ->withType(PropertyType::INTEGER)
        ->actual($category)
        ->createOne();

    $release = ProductPropertyValue::factory()
        ->forProduct($product)
        ->forProperty($property)
        ->published()
        ->createOne()
        ->release;

    $release->delete();

    $request = ['properties' => [['id' => $property->id, 'is_required' => false]]];
    postJson("/api/v1/categories/categories/{$category->id}:bind-properties", $request)
        ->assertOk();

    assertNotSoftDeleted($release);
});

test('hide deactivated attribute', function () {
    $category = Category::factory()->createOne();
    $product = Product::factory()->inCategory($category)->create();
    $property = Property::factory()
        ->withType(PropertyType::STRING)
        ->actual($category)
        ->createOne();

    CategoryPropertyLink::factory()->createFast($category, $property);
    $release = ProductPropertyValue::factory()
        ->forProduct($product)
        ->forProperty($property)
        ->published()
        ->createOne()
        ->release;

    patchJson("/api/v1/categories/properties/{$property->id}", ['is_active' => false])
        ->assertOk();

    assertSoftDeleted($release);
});

test('show activated attribute', function () {
    $category = Category::factory()->createOne();
    $product = Product::factory()->inCategory($category)->create();
    $property = Property::factory()
        ->withType(PropertyType::DOUBLE)
        ->inactive()
        ->actual($category)
        ->createOne();

    CategoryPropertyLink::factory()->createFast($category, $property);
    $release = ProductPropertyValue::factory()
        ->forProduct($product)
        ->forProperty($property)
        ->published()
        ->createOne()
        ->release;

    $release->delete();

    patchJson("/api/v1/categories/properties/{$property->id}", ['is_active' => true])
        ->assertOk();

    assertNotSoftDeleted($release);
});
